"use strict";

var _utils = require("../utils");
var _common = require("../common");
var _AlarmsTrackable = require("./AlarmsTrackable");
function TrackableFactory() {
  this.create = () => ({
    uid: (0, _utils.getUUID)(),
    name: 'Trackable',
    category: 'VEHICLE'
  });
}
function UncertifiedAlarm(time_set, time_clear) {
  this.time_set = time_set;
  this.time_clear = time_clear;
}
function ViolationAlarm(time_set, time_clear, zone_uid = (0, _utils.getUUID)()) {
  this.time_set = time_set;
  this.time_clear = time_clear;
  this.additional_info = {
    zone_uid,
    zone_name: 'ZoneName',
    zone_alarm_level: 1,
    zone_violation_type: 'keep_out'
  };
}
function EmergencyAlarm(time_set, time_clear) {
  this.time_set = time_set;
  this.time_clear = time_clear;
}
function AlarmFactory() {
  this.create = (type, duration) => {
    const time_set = new Date('2021-03-26 12:00:00').toISOString();
    const time_clear = duration ? new Date(new Date(time_set).getTime() + duration).toISOString() : null;
    if (type === _common.ALARM_TYPE.UNCERTIFIED_OPERATION.name) {
      return new UncertifiedAlarm(time_set, time_clear);
    } else if (type === _common.ALARM_TYPE.BUFFER_VIOLATION.name) {
      return new ViolationAlarm(time_set, time_clear);
    } else if (type === _common.ALARM_TYPE.ZONE_VIOLATION.name) {
      return new ViolationAlarm(time_set, time_clear);
    } else if (type === _common.ALARM_TYPE.EMERGENCY_BUTTON.name) {
      return new EmergencyAlarm(time_set, time_clear);
    }
  };
}
const trackableFactory = new TrackableFactory();
const alarmFactory = new AlarmFactory();
describe('AlarmsTrackable.', () => {
  it('fails init without valid arguments', () => {
    const trackable = trackableFactory.create();
    const from = '2021-03-26 12:00:11';
    const to = '2021-03-26 11:00:00';
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable();
    }).toThrow('"trackable" should be provided');
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable(trackable);
    }).toThrow('"from" should be provided');
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable(trackable, 'Invalid date');
    }).toThrow('"from" should be provided');
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable(trackable, from);
    }).toThrow('"to" should be provided');
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable(trackable, from, 'Invalid date');
    }).toThrow('"to" should be provided');
    expect(() => {
      new _AlarmsTrackable.AlarmsTrackable(trackable, from, to);
    }).toThrow('"to" should be greater than "from" argument');
  });
  it('init', () => {
    const trackable = trackableFactory.create();
    const from = '2021-03-26 12:00:00';
    const to = '2021-03-26 13:00:00';
    const alarmTrackable = new _AlarmsTrackable.AlarmsTrackable(trackable, from, to);
    expect(alarmTrackable.from).toEqual(new Date(from).getTime());
    expect(alarmTrackable.to).toEqual(new Date(to).getTime());
    expect(alarmTrackable.name).toEqual(trackable.name);
    expect(alarmTrackable.category).toEqual(trackable.category);
    expect(alarmTrackable.uid).toEqual(trackable.uid);
    expect(alarmTrackable.count).toEqual(0);
    expect(alarmTrackable.zone_violation_duration).toEqual(0);
    expect(alarmTrackable.zone_violations).toEqual([]);
    expect(alarmTrackable.buffer_violation_duration).toEqual(0);
    expect(alarmTrackable.buffer_violations).toEqual([]);
    expect(alarmTrackable.emergency_button).toEqual({
      count: 0,
      accumulated_duration: 0
    });
    expect(alarmTrackable.uncertified_operation).toEqual({
      count: 0,
      accumulated_duration: 0
    });
  });
  it('handle alarms', () => {
    const trackable = trackableFactory.create();
    const from = '2021-03-26 00:00:00';
    const to = '2021-03-27 00:00:00';
    const alarmDuration = 10000;
    const alarmTrackable = new _AlarmsTrackable.AlarmsTrackable(trackable, from, to);
    const bufferAlarm = alarmFactory.create(_common.ALARM_TYPE.BUFFER_VIOLATION.name, alarmDuration);
    const zoneAlarm = alarmFactory.create(_common.ALARM_TYPE.ZONE_VIOLATION.name, alarmDuration);
    const uncertAlarm = alarmFactory.create(_common.ALARM_TYPE.UNCERTIFIED_OPERATION.name, alarmDuration);
    const emergencyAlarm = alarmFactory.create(_common.ALARM_TYPE.EMERGENCY_BUTTON.name, alarmDuration);
    alarmTrackable.emergencyButtonAlarm = emergencyAlarm;
    alarmTrackable.bufferViolationAlarm = bufferAlarm;
    alarmTrackable.zoneViolationAlarm = zoneAlarm;
    alarmTrackable.uncertifiedOpAlarm = uncertAlarm;
    expect(alarmTrackable.count).toEqual(4);
    expect(alarmTrackable.zone_violation_duration).toEqual(alarmDuration);
    expect(alarmTrackable.zone_violations.length).toEqual(1);
    expect(alarmTrackable.buffer_violation_duration).toEqual(alarmDuration);
    expect(alarmTrackable.buffer_violations.length).toEqual(1);
    expect(alarmTrackable.emergency_button).toEqual({
      count: 1,
      accumulated_duration: alarmDuration
    });
    expect(alarmTrackable.uncertified_operation).toEqual({
      count: 1,
      accumulated_duration: alarmDuration
    });
  });
  // todo: Provide other test cases
});