"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _stream = require("stream");
var _utils = require("../utils");
var _common = require("../common");
var _models = require("../models");
var _repository = require("../repository");
var _logger = _interopRequireDefault(require("../logger"));
class Report {
  constructor(from, to) {
    this.trackables = {};
    this.from = new Date(from).getTime();
    this.to = new Date(to).getTime();
  }
  set trackablesMap(trackables) {
    trackables.forEach(trackable => {
      this.trackables[trackable.uid] = new _models.AlarmsTrackable(trackable, this.from, this.to);
    });
  }
  handleAlarm(alarm) {
    if (!this.trackables[alarm.obj_uid]) return;
    if (alarm.type === _common.ALARM_TYPE.ZONE_VIOLATION.name) {
      this.trackables[alarm.obj_uid].zoneViolationAlarm = alarm;
    } else if (alarm.type === _common.ALARM_TYPE.EMERGENCY_BUTTON.name) {
      this.trackables[alarm.obj_uid].emergencyButtonAlarm = alarm;
    } else if (alarm.type === _common.ALARM_TYPE.BUFFER_VIOLATION.name) {
      this.trackables[alarm.obj_uid].bufferViolationAlarm = alarm;
    } else if (alarm.type === _common.ALARM_TYPE.UNCERTIFIED_OPERATION.name) {
      this.trackables[alarm.obj_uid].uncertifiedOpAlarm = alarm;
    }
  }
  get output() {
    return Object.values(this.trackables).filter(trackable => trackable.count);
  }
}
async function getAlarms(options, report) {
  const from = (0, _utils.timeFormatMySql)(options.start);
  const to = (0, _utils.timeFormatMySql)(options.end);
  const sublocationUids = (await _repository.LocationRepo.getSublocations(options.location.uid)).map(sl => sl.uid);
  const alarmsStream = await _repository.AlarmRepo.getAlarmsStream(from, to, sublocationUids);
  const transformStream = new _stream.Transform({
    writableObjectMode: true,
    transform(row, encoding, callback) {
      try {
        report.handleAlarm((0, _utils.formatAlarm)(row));
        callback();
      } catch (err) {
        callback();
      }
    },
    flush(callback) {
      callback();
    }
  });
  alarmsStream.pipe(transformStream);
  return new Promise((resolve, reject) => {
    transformStream.on('data', () => {});
    transformStream.on('end', () => {
      resolve(null);
    });
    transformStream.on('close', () => {
      resolve(null);
    });
    transformStream.on('error', err => {
      reject(err);
    });
  });
}
async function getTrackables(start, end, location) {
  const sublocationUids = (await _repository.LocationRepo.getSublocations(location.uid)).map(sublocation => sublocation.uid);
  return _repository.HeartbeatRepo.getObjects(start, end, sublocationUids);
}
async function combineReport(start, end, location, options, report) {
  report.trackablesMap = await getTrackables(start, end, location);
  return {
    shift: JSON.stringify(location.shift),
    project_name: options.project_name,
    project_uid: options.project_uid,
    type: 'alarms_report',
    site_name: location.name,
    site_uid: location.uid,
    uid: (0, _utils.getUUID)(),
    start,
    end
  };
}
async function main(options) {
  // console.log('[rtls-report][Report Alarms] Report ', JSON.stringify(options));
  for (let i = 0; i < options.locations.length; i += 1) {
    const location = options.locations[i];
    const {
      start,
      end
    } = (0, _utils.getShiftTimeframe)(location.shift, location.timezone);
    const report = new Report(start, end);
    const result = await combineReport(start, end, location, options, report);
    await getAlarms({
      start,
      end,
      location
    }, report);
    result.data = JSON.stringify(report.output);
    try {
      await _repository.ReportRepo.saveDw(result);
      // console.log('[rtls-report][Report Alarms] Report ', result.uid, 'saved in rpp_dw_prod');
    } catch (err) {
      _logger.default.error(err);
      await _repository.ReportRepo.save(result);
      // console.log('[rtls-report][Report Alarms] Report ', result.uid, 'saved in dr2_prod');
    }
  }
}

async function handleMessage(report) {
  try {
    await main(report);
    _logger.default.info('[Report Alarms] Done');
    process.exit(0);
  } catch (err) {
    _logger.default.error(err);
    process.exit(1);
  }
}
process.on('message', handleMessage);
process.on('uncaughtException', reason => {
  _logger.default.error('[Report Alarms]: uncaughtException');
  _logger.default.error('[Report Alarms]: reason ', JSON.stringify(reason));
  process.exit(1);
});
process.on('unhandledRejection', reason => {
  _logger.default.error('[Report Alarms]: unhandledRejection');
  _logger.default.error('[Report Alarms]: reason ', JSON.stringify(reason));
  process.exit(1);
});