"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
var _stream = require("stream");
var _utils = require("../utils");
var _models = require("../models");
var _repository = require("../repository");
var _logger = _interopRequireDefault(require("../logger"));
const REPORT_GRANULARITY = 60 * 60 * 1000;
async function getPositions(options, report) {
  const from = (0, _utils.timeFormatMySql)(options.start);
  const to = (0, _utils.timeFormatMySql)(options.end);
  if (!Array.isArray(report.area_uids) || report.area_uids.length === 0) {
    return Promise.resolve(JSON.stringify({}));
  }
  const positionsStream = await _repository.PositionRepo.getStream(from, to, report.area_uids);
  const transformStream = new _stream.Transform({
    writableObjectMode: true,
    transform(row, encoding, callback) {
      try {
        row.ts = Number(row.ts);
        report.handleTrackablePosition(row);
        callback();
      } catch (err) {
        callback();
      }
    },
    flush(callback) {
      callback();
    }
  });
  positionsStream.pipe(transformStream);
  return new Promise((resolve, reject) => {
    transformStream.on('data', () => {});
    transformStream.on('end', () => {
      resolve(JSON.stringify(report.output));
    });
    transformStream.on('close', () => {
      resolve(JSON.stringify(report.output));
    });
    transformStream.on('error', err => {
      reject(err);
    });
  });
}
class DistanceReport {
  constructor(props) {
    this.ranges = (0, _utils.getRanges)(props.start, props.end, props.granularity, props.reportFirstRangesShift);
    this.step = props.granularity;
    this.reportFirstRangesShift = props.reportFirstRangesShift;
    this.startDate = props.start;
    this.endDate = props.end;
    this.trackables = {};
    this.area_uids = [];
  }
  options() {
    return {
      startDate: this.startDate,
      endDate: this.endDate,
      step: this.step,
      reportFirstRangesShift: this.reportFirstRangesShift,
      ranges: this.ranges,
      distances: this.ranges.map(() => 0)
    };
  }
  set trackablesMap(trackables) {
    trackables.forEach(trackable => {
      this.trackables[trackable.uid] = new _models.TravelDistanceTrackable(trackable, this.options());
    });
  }
  set areas(sublocations) {
    this.area_uids = sublocations.map(sublocation => sublocation.uid);
  }
  handleTrackablePosition(position) {
    if (!this.trackables[position.obj_uid]) return;
    this.trackables[position.obj_uid].handlePosition(position);
  }
  get output() {
    return {
      ranges: this.ranges,
      trackables: Object.values(this.trackables).reduce((memo, tko) => {
        if (tko.totalDistance) {
          memo.push(tko.data);
        }
        return memo;
      }, [])
    };
  }
}
async function getTrackables(start, end, location) {
  const sublocationUids = (await _repository.LocationRepo.getSublocations(location.uid)).map(sublocation => sublocation.uid);
  return _repository.HeartbeatRepo.getObjects(start, end, sublocationUids);
}
async function combineReport(start, end, location, options, report) {
  report.trackablesMap = await getTrackables(start, end, location);
  report.areas = await _repository.LocationRepo.getSublocations(location.uid);
  return {
    shift: JSON.stringify(location.shift),
    project_name: options.project_name,
    project_uid: options.project_uid,
    type: 'distance_report',
    site_name: location.name,
    site_uid: location.uid,
    uid: (0, _utils.getUUID)(),
    start,
    end
  };
}
async function main(options) {
  // console.log('[rtls-report][Report Distance] Report ', JSON.stringify(options));
  for (let i = 0; i < options.locations.length; i += 1) {
    const location = options.locations[i];
    const {
      start,
      end
    } = (0, _utils.getShiftTimeframe)(location.shift, location.timezone);
    const startMinutes = new Date(start).getMinutes();
    const startIsFullHour = startMinutes === 0;
    const reportFirstRangesShift = startIsFullHour ? 0 : (60 - startMinutes) * 60 * 1000;
    const report = new DistanceReport({
      start: new Date(start).getTime(),
      end: new Date(end).getTime(),
      granularity: REPORT_GRANULARITY,
      reportFirstRangesShift
    });
    const result = await combineReport(start, end, location, options, report);
    result.data = await getPositions({
      start,
      end,
      location
    }, report);
    try {
      await _repository.ReportRepo.saveDw(result);
      // console.log('[rtls-report][Report Distance] Report ', result.uid, 'saved in rpp_dw_prod');
    } catch (err) {
      _logger.default.error(err);
      await _repository.ReportRepo.save(result);
      // console.log('[rtls-report][Report Distance] Report ', result.uid, 'saved in dr2_prod');
    }
  }
}

async function handleMessage(report) {
  try {
    await main(report);
    _logger.default.info('[Report Distance] Done');
    process.exit(0);
  } catch (err) {
    _logger.default.error('[Report Distance]: Error');
    _logger.default.error(err);
    process.exit(1);
  }
}
process.on('message', handleMessage);
process.on('uncaughtException', reason => {
  _logger.default.error('[Report Distance]: uncaughtException');
  _logger.default.error('[Report Distance]: reason ', JSON.stringify(reason));
  process.exit(1);
});
process.on('unhandledRejection', reason => {
  _logger.default.error('[Report Distance]: unhandledRejection');
  _logger.default.error('[Report Distance]: reason ', JSON.stringify(reason));
  process.exit(1);
});