"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");
Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.LocationReports = void 0;
exports.createJob = createJob;
exports.locationReports = void 0;
var _cron = require("cron");
var _ReportWorker = _interopRequireDefault(require("./ReportWorker"));
const debug = require('debug')('rtls-reports:LocationReports');
const cron = require('cron-validator');
const EVERY_DAY_MIDNIGHT_CRON = '0 0 * * *';
const DEFAULT_TIMEZONE = 'Etc/GMT';
const DATA_TRANSFER_LATENCY = 5 * 60 * 1000;
function createJob(cronString, locations, timezone, project_uid, project_name) {
  return new _cron.CronJob(cronString, () => {
    debug('[Schedule Report] timezone', timezone);
    debug('[Schedule Report] locations ', locations);
    const options = {
      types: ['attendance_report', 'alarms', 'zone_activity'],
      // removed 'distance'
      project_uid,
      project_name,
      timezone,
      locations
    };
    setTimeout(() => {
      new _ReportWorker.default(options).run();
    }, DATA_TRANSFER_LATENCY);
  }, null, true, timezone);
}
class LocationReports {
  constructor() {
    /**
     * Schedule
     * @type {{
     *      [project_uid]: {
     *          [shiftEndCronString]: {
     *              job: CronJob
     *              locations: {
     *                  uid: String
     *                  shift: {
     *                      start: String
     *                      end: String
     *                  }
     *                  name: String
     *              }
     *          },
     *          byLocationUid: {
     *              [location_uid]: Array<String>
     *          }
     *     }
     * }}
     * @description byLocationUid contains array of cron strings locations[i].shift.end
     */
    this.projectSchedules = {};
  }
  get schedules() {
    return this.projectSchedules;
  }
  addLocation(_location, project) {
    const location = {
      ..._location,
      shift: _location.shift.filter(_shift => _shift.active)
    };
    this.deleteLocation(location, project);
    if (Array.isArray(location.shift) && location.shift.length > 0) {
      for (let i = 0; i < location.shift.length; i += 1) {
        this.addSchedule(location.shift[i], location.uid, location.name, project);
      }
    } else {
      this.addSchedule(undefined, location.uid, location.name, project);
    }
  }

  /**
   * Description
   * @param {Object} _shift
   * @property {String} _shift.end cron string
   * @param {String} uid - location uid
   * @param {String} name - location name
   * @param {Object} project
   * @property {String} project.uid
   * @property {String} project.name
   * @property {String} project.user_params.timezone
   */
  addSchedule(_shift, uid, name, project) {
    let shiftEndCronString = _shift && _shift.end;
    let shift = _shift;
    if (!shiftEndCronString || !cron.isValidCron(shiftEndCronString)) {
      shiftEndCronString = EVERY_DAY_MIDNIGHT_CRON;
    }
    if (!shift) {
      shift = {
        start: EVERY_DAY_MIDNIGHT_CRON,
        end: EVERY_DAY_MIDNIGHT_CRON
      };
    }
    if (!this.projectSchedules[project.uid]) {
      this.projectSchedules[project.uid] = {
        byLocationUid: {},
        [shiftEndCronString]: {
          locations: []
        }
      };
    }
    if (!this.projectSchedules[project.uid][shiftEndCronString]) {
      this.projectSchedules[project.uid][shiftEndCronString] = {
        locations: []
      };
    }
    if (!this.projectSchedules[project.uid].byLocationUid[uid]) {
      this.projectSchedules[project.uid].byLocationUid[uid] = [];
    }
    const isLocationFound = this.projectSchedules[project.uid][shiftEndCronString].locations.find(location => location.uid === uid);
    if (!isLocationFound) {
      this.projectSchedules[project.uid][shiftEndCronString].locations.push({
        uid,
        shift,
        name
      });
      this.projectSchedules[project.uid].byLocationUid[uid].push(shiftEndCronString);
    }
    this.runSchedule(project, shiftEndCronString);
  }
  deleteLocation(location, project) {
    const projectSchedules = this.projectSchedules[project.uid];
    const schedules = projectSchedules && projectSchedules.byLocationUid[location.uid];
    if (!schedules || !Array.isArray(schedules) || schedules.length === 0) return;
    delete projectSchedules.byLocationUid[location.uid];
    for (let i = 0; i < schedules.length; i += 1) {
      this.terminateLocationSchedule(schedules[i], location.uid, project);
    }
  }
  terminateLocationSchedule(schedule, location_uid, project) {
    const projectSchedules = this.projectSchedules[project.uid];
    if (projectSchedules && projectSchedules[schedule].locations.find(options => options.uid === location_uid)) {
      projectSchedules[schedule].job.stop();
      projectSchedules[schedule].locations = projectSchedules[schedule].locations.filter(location => location.uid !== location_uid);
      if (projectSchedules[schedule].locations.length > 0) {
        this.runSchedule(project, schedule);
      } else {
        delete projectSchedules[schedule];
      }
    }
  }

  /**
   * Execute schedule
   * @param project
   * @property {String} project.uid
   * @property {String} project.name
   * @property {String} project.user_params.timezone
   * @param {String} shiftEndCronString
   */
  runSchedule(project, shiftEndCronString) {
    const timezone = project.user_params.timezone || DEFAULT_TIMEZONE;
    this.projectSchedules[project.uid][shiftEndCronString].job && this.projectSchedules[project.uid][shiftEndCronString].job.stop();
    const locations = this.projectSchedules[project.uid][shiftEndCronString].locations.map(location => ({
      ...location,
      timezone
    }));
    this.projectSchedules[project.uid][shiftEndCronString].job = createJob(shiftEndCronString, locations, timezone, project.uid, project.name);
    this.projectSchedules[project.uid][shiftEndCronString].job.start();
  }
  changeProjectTimezone(project) {
    if (!this.projectSchedules[project.uid]) return;
    for (const schedule in this.projectSchedules[project.uid]) {
      if (cron.isValidCron(schedule)) {
        this.runSchedule(project, schedule);
      }
    }
  }
}
exports.LocationReports = LocationReports;
const locationReports = new LocationReports();
exports.locationReports = locationReports;