"use strict";

var _utils = require("../utils");
const {
  LocationReports
} = require('./LocationReports');
const DEFAULT_SCHEDULE = '0 0 * * *';
function ProjectFactory() {
  this.create = timezone => ({
    user_params: {
      timezone: timezone || 'Etc/GMT'
    },
    uid: (0, _utils.getUUID)()
  });
}
function LocationFactory() {
  this.create = (project, shift) => ({
    uid: (0, _utils.getUUID)(),
    project_uid: project && project.uid,
    shift: shift || []
  });
}
const locationFactory = new LocationFactory();
const projectFactory = new ProjectFactory();
let locationReports;
describe('locationReports class', () => {
  beforeEach(() => {
    locationReports = new LocationReports();
  });
  it('should add and delete location in scheduler. default shift', () => {
    const project = projectFactory.create('Europe/Kiev');
    const location = locationFactory.create(project);
    expect(locationReports.schedules).toEqual({});
    locationReports.addLocation(location, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const schedules = projectSchedules.byLocationUid[location.uid];
    const reportOptions = projectSchedules[DEFAULT_SCHEDULE].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(schedules.length).toEqual(1);
    expect(reportOptions[0].uid).toEqual(location.uid);
    expect(reportOptions[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
    locationReports.deleteLocation(location, project);
    expect(locationReports.schedules[project.uid]).toEqual({
      byLocationUid: {}
    });
  });
  it('should add and delete location in scheduler. custom shift', () => {
    const shift = [{
      start: '0 0 * * *',
      end: '0 10 * * *',
      active: true
    }, {
      start: '0 0 * * *',
      end: '0 20 * * *',
      active: true
    }];
    const project = projectFactory.create('Europe/Kiev');
    const location = locationFactory.create(project, shift);
    expect(locationReports.schedules).toEqual({});
    locationReports.addLocation(location, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const schedules = projectSchedules.byLocationUid[location.uid];
    const reportOneOptions = projectSchedules[shift[0].end].locations;
    const reportTwoOptions = projectSchedules[shift[1].end].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(schedules.length).toEqual(2);
    expect(reportOneOptions[0].uid).toEqual(location.uid);
    expect(reportOneOptions[0].shift).toEqual(shift[0]);
    expect(reportTwoOptions[0].uid).toEqual(location.uid);
    expect(reportTwoOptions[0].shift).toEqual(shift[1]);
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', shift[0].end, shift[1].end]);
    locationReports.deleteLocation(location, project);
    expect(locationReports.schedules[project.uid]).toEqual({
      byLocationUid: {}
    });
  });
  it('should update location in scheduler. default shift', () => {
    const shift = [{
      start: '0 0 * * *',
      end: '0 10 * * *',
      active: true
    }, {
      start: '0 0 * * *',
      end: '0 20 * * *',
      active: true
    }];
    const project = projectFactory.create('Europe/Kiev');
    const location = locationFactory.create(project);
    locationReports.addLocation(location, project);
    location.shift = shift;
    locationReports.addLocation(location, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const schedules = projectSchedules.byLocationUid[location.uid];
    const reportOneOptions = projectSchedules[shift[0].end].locations;
    const reportTwoOptions = projectSchedules[shift[1].end].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(schedules.length).toEqual(2);
    expect(reportOneOptions[0].uid).toEqual(location.uid);
    expect(reportOneOptions[0].shift).toEqual(shift[0]);
    expect(reportTwoOptions[0].uid).toEqual(location.uid);
    expect(reportTwoOptions[0].shift).toEqual(shift[1]);
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', shift[0].end, shift[1].end]);
  });
  it('should ignore location with inactive schedule', () => {
    const shift = [{
      start: '0 0 * * *',
      end: '0 10 * * *',
      active: false
    }, {
      start: '0 0 * * *',
      end: '0 20 * * *',
      active: false
    }];
    const project = projectFactory.create('Europe/Kiev');
    const location = locationFactory.create(project);
    locationReports.addLocation(location, project);
    location.shift = shift;
    locationReports.addLocation(location, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const schedules = projectSchedules.byLocationUid[location.uid];
    const reportOneOptions = projectSchedules[DEFAULT_SCHEDULE].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(schedules.length).toEqual(1);
    expect(reportOneOptions[0].uid).toEqual(location.uid);
    expect(reportOneOptions[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
  });
  it('should handle two location in scheduler. default shift', () => {
    const project = projectFactory.create('Europe/Kiev');
    const locationOne = locationFactory.create(project);
    const locationTwo = locationFactory.create(project);
    locationReports.addLocation(locationOne, project);
    locationReports.addLocation(locationTwo, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const reportOptions = projectSchedules[DEFAULT_SCHEDULE].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(reportOptions.length).toEqual(2);
    expect(reportOptions[0].uid).toEqual(locationOne.uid);
    expect(reportOptions[1].uid).toEqual(locationTwo.uid);
    expect(reportOptions[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(reportOptions[1].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
  });
  it('should handle two location in scheduler. different shifts', () => {
    const shift = [{
      start: '0 0 * * *',
      end: '0 10 * * *',
      active: true
    }, {
      start: '0 0 * * *',
      end: '0 20 * * *',
      active: true
    }];
    const project = projectFactory.create('Europe/Kiev');
    const locationOne = locationFactory.create(project);
    const locationTwo = locationFactory.create(project, shift);
    locationReports.addLocation(locationOne, project);
    locationReports.addLocation(locationTwo, project);
    const projectSchedules = locationReports.schedules[project.uid];
    const reportOptionsOne = projectSchedules[DEFAULT_SCHEDULE].locations;
    const reportOptionsTwo = projectSchedules[shift[0].end].locations;
    const reportOptionsThree = projectSchedules[shift[1].end].locations;
    expect(locationReports.schedules).toHaveProperty(project.uid);
    expect(reportOptionsOne.length).toEqual(1);
    expect(reportOptionsTwo.length).toEqual(1);
    expect(reportOptionsThree.length).toEqual(1);
    expect(reportOptionsOne[0].uid).toEqual(locationOne.uid);
    expect(reportOptionsTwo[0].uid).toEqual(locationTwo.uid);
    expect(reportOptionsThree[0].uid).toEqual(locationTwo.uid);
    expect(reportOptionsOne[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(reportOptionsTwo[0].shift).toEqual(shift[0]);
    expect(reportOptionsThree[0].shift).toEqual(shift[1]);
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE, shift[0].end, shift[1].end]);
  });
  it('should handle two location two projects in scheduler. default shift', () => {
    const projectOne = projectFactory.create('Europe/Kiev');
    const locationOne = locationFactory.create(projectOne);
    const projectTwo = projectFactory.create('Europe/Kiev');
    const locationTwo = locationFactory.create(projectTwo);
    expect(locationReports.schedules).toEqual({});
    locationReports.addLocation(locationOne, projectOne);
    locationReports.addLocation(locationTwo, projectTwo);
    expect(Object.keys(locationReports.schedules)).toEqual([projectOne.uid, projectTwo.uid]);

    // project One
    const projectSchedulesOne = locationReports.schedules[projectOne.uid];
    const schedulesOne = projectSchedulesOne.byLocationUid[locationOne.uid];
    const reportOptionsOne = projectSchedulesOne[DEFAULT_SCHEDULE].locations;
    expect(schedulesOne.length).toEqual(1);
    expect(reportOptionsOne[0].uid).toEqual(locationOne.uid);
    expect(reportOptionsOne[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedulesOne)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);

    // project Two
    const projectSchedulesTwo = locationReports.schedules[projectTwo.uid];
    const schedulesTwo = projectSchedulesTwo.byLocationUid[locationTwo.uid];
    const reportOptionsTwo = projectSchedulesTwo[DEFAULT_SCHEDULE].locations;
    expect(schedulesTwo.length).toEqual(1);
    expect(reportOptionsTwo[0].uid).toEqual(locationTwo.uid);
    expect(reportOptionsTwo[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedulesTwo)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
  });
  it('should remove one of two locations in scheduler. default shift', () => {
    const projectOne = projectFactory.create('Europe/Kiev');
    const locationOne = locationFactory.create(projectOne);
    const locationTwo = locationFactory.create(projectOne);
    expect(locationReports.schedules).toEqual({});
    locationReports.addLocation(locationOne, projectOne);
    locationReports.addLocation(locationTwo, projectOne);
    locationReports.deleteLocation(locationTwo, projectOne);
    expect(locationReports.schedules).toHaveProperty(projectOne.uid);
    const projectSchedules = locationReports.schedules[projectOne.uid];
    const schedules = projectSchedules.byLocationUid[locationOne.uid];
    const reportOptions = projectSchedules[DEFAULT_SCHEDULE].locations;
    expect(schedules.length).toEqual(1);
    expect(reportOptions.length).toEqual(1);
    expect(reportOptions[0].uid).toEqual(locationOne.uid);
    expect(reportOptions[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
  });
  it('should remove one of two locations in scheduler. different shifts', () => {
    const shift = [{
      start: '0 0 * * *',
      end: '0 10 * * *',
      active: true
    }, {
      start: '0 0 * * *',
      end: '0 20 * * *',
      active: true
    }];
    const projectOne = projectFactory.create('Europe/Kiev');
    const locationOne = locationFactory.create(projectOne);
    const locationTwo = locationFactory.create(projectOne, shift);
    expect(locationReports.schedules).toEqual({});
    locationReports.addLocation(locationOne, projectOne);
    locationReports.addLocation(locationTwo, projectOne);
    locationReports.deleteLocation(locationTwo, projectOne);
    expect(locationReports.schedules).toHaveProperty(projectOne.uid);
    const projectSchedules = locationReports.schedules[projectOne.uid];
    const schedules = projectSchedules.byLocationUid[locationOne.uid];
    const reportOptions = projectSchedules[DEFAULT_SCHEDULE].locations;
    expect(schedules.length).toEqual(1);
    expect(reportOptions.length).toEqual(1);
    expect(reportOptions[0].uid).toEqual(locationOne.uid);
    expect(reportOptions[0].shift).toEqual({
      start: DEFAULT_SCHEDULE,
      end: DEFAULT_SCHEDULE
    });
    expect(Object.keys(projectSchedules)).toEqual(['byLocationUid', DEFAULT_SCHEDULE]);
  });
});