import { Transform } from 'stream';
import { getShiftTimeframe, formatAlarm, timeFormatMySql, getUUID as UID } from '../utils';
import { ALARM_TYPE } from '../common';
import { AlarmsTrackable } from '../models';
import { LocationRepo, HeartbeatRepo, ReportRepo, AlarmRepo } from '../repository';
import logger from '../logger';

class Report {
    constructor(from, to) {
        this.trackables = {};
        this.from = new Date(from).getTime();
        this.to = new Date(to).getTime();
    }

    set trackablesMap(trackables) {
        trackables.forEach((trackable) => {
            this.trackables[trackable.uid] = new AlarmsTrackable(trackable, this.from, this.to);
        });
    }

    handleAlarm(alarm) {
        if (!this.trackables[alarm.obj_uid]) return;

        if (alarm.type === ALARM_TYPE.ZONE_VIOLATION.name) {
            this.trackables[alarm.obj_uid].zoneViolationAlarm = alarm;
        } else if (alarm.type === ALARM_TYPE.EMERGENCY_BUTTON.name) {
            this.trackables[alarm.obj_uid].emergencyButtonAlarm = alarm;
        } else if (alarm.type === ALARM_TYPE.BUFFER_VIOLATION.name) {
            this.trackables[alarm.obj_uid].bufferViolationAlarm = alarm;
        } else if (alarm.type === ALARM_TYPE.UNCERTIFIED_OPERATION.name) {
            this.trackables[alarm.obj_uid].uncertifiedOpAlarm = alarm;
        }
    }

    get output() {
        return Object.values(this.trackables).filter((trackable) => trackable.count);
    }
}

async function getAlarms(options, report) {
    const from = timeFormatMySql(options.start);
    const to = timeFormatMySql(options.end);
    const sublocationUids = (await LocationRepo.getSublocations(options.location.uid)).map((sl) => sl.uid);
    const alarmsStream = await AlarmRepo.getAlarmsStream(from, to, sublocationUids);

    const transformStream = new Transform({
        writableObjectMode: true,
        transform(row, encoding, callback) {
            try {
                report.handleAlarm(formatAlarm(row));
                callback();
            } catch (err) {
                callback();
            }
        },
        flush(callback) {
            callback();
        },
    });
    alarmsStream.pipe(transformStream);

    return new Promise((resolve, reject) => {
        transformStream.on('data', () => {});
        transformStream.on('end', () => {
            resolve(null);
        });
        transformStream.on('close', () => {
            resolve(null);
        });
        transformStream.on('error', (err) => {
            reject(err);
        });
    });
}

async function getTrackables(start, end, location) {
    const sublocationUids = (await LocationRepo.getSublocations(location.uid)).map((sublocation) => sublocation.uid);
    return HeartbeatRepo.getObjects(start, end, sublocationUids);
}

async function combineReport(start, end, location, options, report) {
    report.trackablesMap = await getTrackables(start, end, location);

    return {
        shift: JSON.stringify(location.shift),
        project_name: options.project_name,
        project_uid: options.project_uid,
        type: 'alarms_report',
        site_name: location.name,
        site_uid: location.uid,
        uid: UID(),
        start,
        end,
    };
}

async function main(options) {
    // console.log('[rtls-report][Report Alarms] Report ', JSON.stringify(options));
    for (let i = 0; i < options.locations.length; i += 1) {
        const location = options.locations[i];
        const { start, end } = getShiftTimeframe(location.shift, location.timezone);
        const report = new Report(start, end);

        const result = await combineReport(start, end, location, options, report);
        await getAlarms({ start, end, location }, report);
        result.data = JSON.stringify(report.output);

        try {
            await ReportRepo.saveDw(result);
            // console.log('[rtls-report][Report Alarms] Report ', result.uid, 'saved in rpp_dw_prod');
        } catch (err) {
            logger.error(err);
            await ReportRepo.save(result);
            // console.log('[rtls-report][Report Alarms] Report ', result.uid, 'saved in dr2_prod');
        }
    }
}

async function handleMessage(report) {
    try {
        await main(report);
        logger.info('[Report Alarms] Done');
        process.exit(0);
    } catch (err) {
        logger.error(err);
        process.exit(1);
    }
}

process.on('message', handleMessage);

process.on('uncaughtException', (reason) => {
    logger.error('[Report Alarms]: uncaughtException');
    logger.error('[Report Alarms]: reason ', JSON.stringify(reason));
    process.exit(1);
});

process.on('unhandledRejection', (reason) => {
    logger.error('[Report Alarms]: unhandledRejection');
    logger.error('[Report Alarms]: reason ', JSON.stringify(reason));
    process.exit(1);
});
