import { Transform } from 'stream';
import { getUUID as UID, timeFormatMySql, getShiftTimeframe, getRanges } from '../utils';
import { TravelDistanceTrackable } from '../models';
import { HeartbeatRepo, LocationRepo, PositionRepo, ReportRepo } from '../repository';
import logger from '../logger';

const REPORT_GRANULARITY = 60 * 60 * 1000;

async function getPositions(options, report) {
    const from = timeFormatMySql(options.start);
    const to = timeFormatMySql(options.end);
    if (!Array.isArray(report.area_uids) || report.area_uids.length === 0) {
        return Promise.resolve(JSON.stringify({}));
    }

    const positionsStream = await PositionRepo.getStream(from, to, report.area_uids);
    const transformStream = new Transform({
        writableObjectMode: true,
        transform(row, encoding, callback) {
            try {
                row.ts = Number(row.ts);
                report.handleTrackablePosition(row);
                callback();
            } catch (err) {
                callback();
            }
        },
        flush(callback) {
            callback();
        },
    });

    positionsStream.pipe(transformStream);

    return new Promise((resolve, reject) => {
        transformStream.on('data', () => {});
        transformStream.on('end', () => {
            resolve(JSON.stringify(report.output));
        });
        transformStream.on('close', () => {
            resolve(JSON.stringify(report.output));
        });
        transformStream.on('error', (err) => {
            reject(err);
        });
    });
}

class DistanceReport {
    constructor(props) {
        this.ranges = getRanges(props.start, props.end, props.granularity, props.reportFirstRangesShift);
        this.step = props.granularity;
        this.reportFirstRangesShift = props.reportFirstRangesShift;
        this.startDate = props.start;
        this.endDate = props.end;
        this.trackables = {};
        this.area_uids = [];
    }

    options() {
        return {
            startDate: this.startDate,
            endDate: this.endDate,
            step: this.step,
            reportFirstRangesShift: this.reportFirstRangesShift,
            ranges: this.ranges,
            distances: this.ranges.map(() => 0),
        };
    }

    set trackablesMap(trackables) {
        trackables.forEach((trackable) => {
            this.trackables[trackable.uid] = new TravelDistanceTrackable(trackable, this.options());
        });
    }

    set areas(sublocations) {
        this.area_uids = sublocations.map((sublocation) => sublocation.uid);
    }

    handleTrackablePosition(position) {
        if (!this.trackables[position.obj_uid]) return;
        this.trackables[position.obj_uid].handlePosition(position);
    }

    get output() {
        return {
            ranges: this.ranges,
            trackables: Object.values(this.trackables).reduce((memo, tko) => {
                if (tko.totalDistance) {
                    memo.push(tko.data);
                }
                return memo;
            }, []),
        };
    }
}

async function getTrackables(start, end, location) {
    const sublocationUids = (await LocationRepo.getSublocations(location.uid)).map((sublocation) => sublocation.uid);
    return HeartbeatRepo.getObjects(start, end, sublocationUids);
}

async function combineReport(start, end, location, options, report) {
    report.trackablesMap = await getTrackables(start, end, location);
    report.areas = await LocationRepo.getSublocations(location.uid);

    return {
        shift: JSON.stringify(location.shift),
        project_name: options.project_name,
        project_uid: options.project_uid,
        type: 'distance_report',
        site_name: location.name,
        site_uid: location.uid,
        uid: UID(),
        start,
        end,
    };
}

async function main(options) {
    // console.log('[rtls-report][Report Distance] Report ', JSON.stringify(options));
    for (let i = 0; i < options.locations.length; i += 1) {
        const location = options.locations[i];
        const { start, end } = getShiftTimeframe(location.shift, location.timezone);
        const startMinutes = new Date(start).getMinutes();
        const startIsFullHour = startMinutes === 0;
        const reportFirstRangesShift = startIsFullHour ? 0 : (60 - startMinutes) * 60 * 1000;

        const report = new DistanceReport({
            start: new Date(start).getTime(),
            end: new Date(end).getTime(),
            granularity: REPORT_GRANULARITY,
            reportFirstRangesShift,
        });

        const result = await combineReport(start, end, location, options, report);
        result.data = await getPositions({ start, end, location }, report);

        try {
            await ReportRepo.saveDw(result);
            // console.log('[rtls-report][Report Distance] Report ', result.uid, 'saved in rpp_dw_prod');
        } catch (err) {
            logger.error(err);
            await ReportRepo.save(result);
            // console.log('[rtls-report][Report Distance] Report ', result.uid, 'saved in dr2_prod');
        }
    }
}

async function handleMessage(report) {
    try {
        await main(report);
        logger.info('[Report Distance] Done');
        process.exit(0);
    } catch (err) {
        logger.error('[Report Distance]: Error');
        logger.error(err);
        process.exit(1);
    }
}

process.on('message', handleMessage);

process.on('uncaughtException', (reason) => {
    logger.error('[Report Distance]: uncaughtException');
    logger.error('[Report Distance]: reason ', JSON.stringify(reason));
    process.exit(1);
});

process.on('unhandledRejection', (reason) => {
    logger.error('[Report Distance]: unhandledRejection');
    logger.error('[Report Distance]: reason ', JSON.stringify(reason));
    process.exit(1);
});
