import { Transform } from 'stream';
import { getShiftTimeframe, timeFormatMySql, getUUID as UID } from '../utils';
import { HeartbeatRepo, LocationRepo, EventRepo, ReportRepo } from '../repository';
import { ZoneActivityTrackable } from '../models';
import logger from '../logger';

class Report {
    constructor(from, to) {
        this.trackables = {};
        this.area_uids = [];
        this.from = new Date(from).getTime();
        this.to = new Date(to).getTime();
    }

    set trackablesMap(trackables) {
        trackables.forEach((trackable) => {
            this.trackables[trackable.uid] = new ZoneActivityTrackable(trackable, this.from, this.to);
        });
    }

    set areas(sublocations) {
        this.area_uids = sublocations.map((sublocation) => sublocation.uid);
    }

    handleEvent(event) {
        if (!this.trackables[event.trackable_uid]) return;
        this.trackables[event.trackable_uid].zoneEntry = event;
    }

    get output() {
        return {
            area_uids: this.area_uids,
            trackables: Object.values(this.trackables).filter((trackable) => trackable.count),
        };
    }
}

async function getZoneEvents(options, report) {
    const from = timeFormatMySql(options.start);
    const to = timeFormatMySql(options.end);

    if (!Array.isArray(report.area_uids) || report.area_uids.length === 0) {
        return Promise.resolve(JSON.stringify({}));
    }

    const zoneEventsStream = await EventRepo.getZoneEventsStream(from, to, report.area_uids);
    const transformStream = new Transform({
        writableObjectMode: true,
        transform(row, encoding, callback) {
            try {
                report.handleEvent(row);
                callback();
            } catch (err) {
                callback();
            }
        },
        flush(callback) {
            callback();
        },
    });

    zoneEventsStream.pipe(transformStream);

    return new Promise((resolve, reject) => {
        transformStream.on('data', () => {});
        transformStream.on('end', () => {
            resolve(null);
        });
        transformStream.on('close', () => {
            resolve(null);
        });
        transformStream.on('error', (err) => {
            reject(err);
        });
    });
}

async function getTrackables(start, end, location) {
    const sublocationUids = (await LocationRepo.getSublocations(location.uid)).map((sublocation) => sublocation.uid);
    return HeartbeatRepo.getObjects(start, end, sublocationUids);
}

async function combineReport(start, end, location, options, report) {
    report.trackablesMap = await getTrackables(start, end, location);
    report.areas = await LocationRepo.getSublocations(location.uid);

    return {
        shift: JSON.stringify(location.shift),
        project_name: options.project_name,
        project_uid: options.project_uid,
        type: 'zone_activity_report',
        site_name: location.name,
        site_uid: location.uid,
        uid: UID(),
        start,
        end,
    };
}

async function main(options) {
    // console.log('[rtls-report][Report Zone Activity] Report ', JSON.stringify(options));
    for (let i = 0; i < options.locations.length; i += 1) {
        const location = options.locations[i];
        const { start, end } = getShiftTimeframe(location.shift, location.timezone);
        const report = new Report(start, end);

        const result = await combineReport(start, end, location, options, report);
        await getZoneEvents({ start, end, location }, report);
        result.data = JSON.stringify(report.output);

        try {
            await ReportRepo.saveDw(result);
            // console.log('[rtls-report][Report Zone Activity] Report ', result.uid, 'saved in rpp_dw_prod');
        } catch (err) {
            logger.error(err);
            await ReportRepo.save(result);
            // console.log('[rtls-report][Report Zone Activity] Report ', result.uid, 'saved in dr2_prod');
        }
    }
}

async function handleMessage(report) {
    try {
        await main(report);
        logger.info('[Report Zone Activity] Done');
        process.exit(0);
    } catch (err) {
        logger.error('[Report Zone Activity]: Error');
        logger.error(err);
        process.exit(1);
    }
}

process.on('message', handleMessage);

process.on('uncaughtException', (reason) => {
    logger.error('[Report Zone Activity]: uncaughtException');
    logger.error('[Report Zone Activity]: reason ', JSON.stringify(reason));
    process.exit(1);
});

process.on('unhandledRejection', (reason) => {
    logger.error('[Report Zone Activity]: unhandledRejection');
    logger.error('[Report Zone Activity]: reason ', JSON.stringify(reason));
    process.exit(1);
});
