const debug = require('debug')('rtls-reports:puller');
import { DB_PULL_INTERVAL } from '../common';
import { ProjectRepo, LocationRepo } from '../repository';
import { locationReports } from './LocationReports';
import { formatLocationArray, parseSafeNull } from '../utils';

const ProjectMap = new Map();
const LocationMap = new Map();

class Puller {
    constructor() {
        setInterval(this.pullProjects.bind(this), DB_PULL_INTERVAL);
    }

    timezoneChanged(newProject) {
        const existedProject = ProjectMap.get(newProject.uid);
        if (existedProject) {
            const existedTimezone = existedProject.user_params && existedProject.user_params.timezone;
            const newTimezone = newProject.user_params && newProject.user_params.timezone;
            return existedTimezone !== newTimezone;
        }
        return false;
    }

    async pullProjects() {
        debug('pulling Projects ');
        const projects = (await ProjectRepo.getList()).map((row) => {
            if (row.user_params) {
                row.user_params = parseSafeNull(row.user_params);
            }
            return row;
        });

        projects.forEach((project) => {
            if (this.timezoneChanged(project)) {
                locationReports.changeProjectTimezone(project);
            }
            ProjectMap.set(project.uid, project);
        });
        ProjectMap.forEach((existingProject) => {
            if (
                !projects.find((project) => {
                    return project.uid === existingProject.uid;
                })
            ) {
                ProjectMap.delete(existingProject.uid);
            }
        });

        const locations = formatLocationArray(await LocationRepo.getList());
        // debug('view_locations:', locations);
        locations.forEach((location) => {
            const existingLocation = LocationMap.get(location.uid);
            if (existingLocation) {
                if (existingLocation.updated_at.getTime() !== location.updated_at.getTime()) {
                    debug('Location updated.', location);
                    this.setLocation(location);
                }
            } else {
                debug('Location added.');
                this.setLocation(location);
            }
        });
        LocationMap.forEach((existingLocation) => {
            if (
                !locations.find((location) => {
                    return location.uid === existingLocation.uid;
                })
            ) {
                debug('Location deleted.');
                this.deleteLocation(existingLocation);
            }
        });
    }

    setLocation(location) {
        if (!location.uid) {
            return false;
        }
        const project = ProjectMap.get(location.project_uid);
        if (project) {
            locationReports.addLocation(location, project);
        }
        return LocationMap.set(location.uid, location);
    }

    deleteLocation(location) {
        if (!location.uid) return false;

        const project = ProjectMap.get(location.project_uid);
        if (project) {
            locationReports.deleteLocation(location, project);
        }
        LocationMap.delete(location.uid);
        return !!location;
    }
}

module.exports = {
    Puller,
};
